from AnimeCrawler.command import CrawlerCommands
from AnimeCrawler.mhyyy import AnimeSpider, Downloader
from AnimeCrawler.utils import (
    base64_decode,
    folder_path,
    get_video_path,
    merge_ts2mp4,
    unescape,
    write,
)



import asyncio

import aiohttp
import tqdm.asyncio

from AnimeCrawler.utils import write


class Downloader:
    session = None

    def __init__(self, urls):
        self.urls = urls

    @property
    def current_sesion(self):
        if not self.session:
            self.session = aiohttp.ClientSession(
                connector=aiohttp.TCPConnector(limit=40, verify_ssl=False)
            )
        return self.session

    async def get_ts_file(
        self,
        title: str,
        url: str,
        error_times: int = 1,
    ):
        resp = await self.current_sesion.get(
            url=url,
            headers={'User-Agent': 'Mozilla/5.0', ' Transfer-Encoding': 'chunked'},
        )
        try:
            text = await resp.content.read()
            return (text, title)
        except aiohttp.ClientPayloadError as e:  # 报错时重新下载
            if error_times == 3:
                raise Warning(f'下载{title}.ts时发生错误') from e
            print(f'下载{title}.ts时发生错误，正在重试第{error_times}次')
            return await self.get_ts_file(title, url, error_times + 1)
        except Exception as e:
            print(f'下载{title}.ts时发生{e}')
            await asyncio.sleep(0)
        finally:
            resp.close()

    async def close_session(self):
        if self.session:
            await self.session.close()

    async def download_ts_files(self, path, episodes):
        tasks = [
            self.get_ts_file(str(index).zfill(4), url)
            for index, url in enumerate(self.urls)
        ]
        for task in tqdm.asyncio.tqdm.as_completed(tasks, desc=f"正在下载第{episodes}集视频"):
            result = await task
            text_1, title = (
                (b'error', 'error') if result is None else result
            )  # result为空时返回'error'
            await write(path, text_1, title, suffix='ts', mode='wb')
        await self.close_session()



from AnimeCrawler.utils import (
    base64_decode,
    folder_path,
    get_video_path,
    merge_ts2mp4,
    unescape,
    write,
)

from .downloader import Downloader
from .spider import AnimeSpider

__version__ = 'v0.1.0_dev_3'



import ctypes.wintypes
from pathlib import Path
from typing import Union

import aiofiles


def folder_path(folder_path):
    '''判定文件夹是否存在，不存在就创建

    Args:
        folder_path (Path): 文件夹路径

    Returns:
        Path: 文件夹路径
    '''
    if not folder_path.exists():
        folder_path.mkdir()
    return folder_path


def get_video_path(path_id=14):
    '''获取用户视频文件夹路径

    Args:
        path_id (int, optional): 用户视频文件夹路径在Windows系统中的编号，不可动. Defaults to 14.

    Returns:
        str: 用户视频文件夹路径
    '''
    buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
    ctypes.windll.shell32.SHGetFolderPathW(None, path_id, None, 0, buf)
    return buf.value


async def write(
    path: Union[str, Path],
    text: str,
    title: Union[str, int],
    suffix: str = 'txt',
    mode: str = 'w',
):
    '''使用aiofiles，在文件夹下存储文件

    Args:
        path (Path): 文件夹路径
        text (str): 文件内容
        title (str): 文件标题
        suffix (str, optional): 文件后缀. Defaults to 'txt'.
        mode (str, optional): 写入模式，

        详情请见
        https://docs.python.org/zh-cn/3.10/library/functions.html#open.

        Defaults to 'a'.
    '''
    folder = folder_path(path)
    async with aiofiles.open(folder / f'{title}.{suffix}', mode) as fp:
        await fp.write(text)


async def merge_ts2mp4(folder_path: Path, episodes: int = None, del_ts: bool = False):
    '''将ts文件合并成mp4

    Args:
        folder_path (Path): 文件夹路径，里面应有ts文件
        episodes (int): 动漫集数. Defaults to None.
    '''
    print(del_ts)
    for file_path in folder_path.iterdir():
        if file_path.suffix == '.ts':
            with open(file_path, 'rb') as f1:
                with open(folder_path / f"第{episodes}集.mp4", 'ab') as f2:
                    f2.write(f1.read())
            if del_ts:
                file_path.unlink()



import html
import re
import urllib.parse
from base64 import b64decode


def base64_decode(string: str) -> str:
    '''对base64.b64decode()的包装

    Arguments:
        string -- 要解码的字符串

    Returns:
        解码后的字符串
    '''
    byte = bytes(string, 'utf-8')
    string = b64decode(byte).decode()
    return string


def unescape(string) -> str:
    '''对html.unescape()的包装

    Arguments:
        string -- 要解码的字符串

    Returns
        解码后的字符串'''
    string = urllib.parse.unquote(string)
    quoted = html.unescape(string).encode().decode('utf-8')
    # 转成中文
    return re.sub(
        r'%u([a-fA-F0-9]{4}|[a-fA-F0-9]{2})',
        lambda m: chr(int(m.group(1), 16)),
        quoted,
    )



import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="AnimeCrawler",
    version="v0.1.1",
    author="Senvlin",
    author_email="2994515402@qq.com",
    description="一个可免费下载动漫的爬虫",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Senvlin/AnimeCrawler",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'ruia',
        'tqdm',
        'aiofiles',
    ],
    entry_points={
        'console_scripts': [
            'animecrawler = AnimeCrawler.command.run:main',
        ],
    },
    python_requires='>=3.8',
)



import argparse
import re

from AnimeCrawler.mhyyy.spider import AnimeSpider


class CrawlerCommands:
    '''https://docs.python.org/zh-cn/3/library/argparse.html#quick-links-for-add-argument'''

    def is_url(self, string):
        '''解析是否为url

        Args:
            string (str): 要解析的字符串

        Returns:
            bool: 用于判断
        '''
        pattern = re.compile(
            r'^(?:http|ftp)s?://'  # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
            r'localhost|'  # localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
            r'(?::\d+)?'  # optional port
            r'(?:/?|[/?]\S+)$',
            re.IGNORECASE,
        )
        return bool(re.search(pattern, string))

    def get_parser(self):
        parser = argparse.ArgumentParser(
            prog='AnimeCrawler',
            usage='%(prog)s <Commands> [Options]',
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description='* AnimeCrawler v0.1.1 - 一个可免费下载动漫的爬虫\n* Repo: https://github.com/Senvlin/AnimeCrawler',
            epilog='Had Issues? Go To -> https://github.com/Senvlin/AnimeCrawler/issues',
            add_help=False,  # 在这里是为了自定义help文档所以禁用了
        )
        group1 = parser.add_argument_group('Commands', '主要命令')
        group1.add_argument('download', help="下载动漫", nargs='?', default=False)
        group1.add_argument('search', help="搜索动漫，未实现", action='store_true')
        group2 = parser.add_argument_group('Required', '必填参数')
        group2.add_argument("-t", "--title", metavar='Title', help="动漫名称")
        group3 = parser.add_argument_group('Options', '可选命令')
        group3.add_argument("-u", "--url", metavar='URL', help="动漫第一集的url")
        group3.add_argument(
            "--del_ts", dest='can_del_ts', help="删除ts文件", action='store_true'
        )
        group3.add_argument(
            "--test", dest='is_test', help="开启Test模式", action='store_true'
        )
        group3.add_argument("-h", dest='help', help="获取帮助，并退出程序", action='store_true')
        return parser

    def parse(self, test_list: list = None):
        parser = self.get_parser()
        parse = parser.parse_args(test_list)
        if parse.help:
            parser.print_help()
            parser.exit()
        if parse.download != 'download' and parse.search != 'search':
            if parse.is_test:
                return 'null_command'
            raise ValueError(f'无效的命令 {parse.download}')
        elif parse.search == 'search':
            if parse.is_test:
                return 'have not supported \'search\' command'
            raise NotImplementedError(f'未支持的命令 {parse.search}')
        elif parse.title is None:
            if parse.is_test:
                return 'null_title'
            raise ValueError(f'{parse.title} 为空')
        elif not self.is_url(parse.url):
            if parse.is_test:
                return 'is_not_url'
            raise ValueError(f'{parse.url} 不为合法的url')
        else:
            if parse.is_test:
                return 'OK'
            AnimeSpider.init(parse.title, parse.url, parse.can_del_ts).start()


def main():
    CrawlerCommands().parse()



import re
from pathlib import Path

from ruia import Item, Response, Spider, TextField

from AnimeCrawler.utils import (
    base64_decode,
    folder_path,
    get_video_path,
    merge_ts2mp4,
    unescape,
    write,
)

from .downloader import Downloader


class AnimeItem(Item):
    target_item = TextField(xpath_select='//div[@class="player-box-main"]')
    profile = TextField(xpath_select='//div/script[@type="text/javascript"]')
    episodes = None
    _base_m3u8_url = None
    mixed_m3u8_url = None


class AnimeSpider(Spider):
    _base_ts_url = None
    _mixed_m3u8 = None
    headers = {'User-Agent': 'Mozilla/5.0'}

    @classmethod
    def init(cls, anime_title: str, start_urls: str, del_ts: bool = False):
        '''初始化爬虫

        Args:
            anime_title (str): 动漫标题，用于取文件夹的名称，利于管理动漫
            start_urls (str): 第一页的网址

        Returns:
            cls: 为了链式调用返回了cls
        '''
        cls.start_urls = [start_urls]
        cls.del_ts = del_ts
        video_path = Path(get_video_path()) / anime_title  # 在项目目录下存储
        cls.PATH = folder_path(video_path)
        return cls

    async def _mixed_m3u8_url_parse(self, index_m3u8_url: str, item: AnimeItem) -> None:
        resp = await self.request(index_m3u8_url).fetch()
        text = await resp.text()
        if self._mixed_m3u8 is None:
            self._mixed_m3u8 = text.split('\n')[-1]
        item.mixed_m3u8_url = item._base_m3u8_url + self._mixed_m3u8

    def _parse_mixed_m3u8(self, item: AnimeItem):
        '''解析mixed.m3u8文件，获得ts文件下载地址

        Returns:
            str：ts_url
        '''
        base_ts_file = item.mixed_m3u8_url[:-10]
        with open(self.PATH / f'{item.episodes}\\mixed.m3u8', 'r') as fp:
            for i in fp:
                if '#' not in i:
                    yield base_ts_file + i

    async def have_next_page(self, link_next: str):
        # 当有下一页时
        link_next = link_next.replace('\\', '')
        return self.request(
            'https://www.mhyyy.com' + link_next,
            callback=self.parse,
            headers=self.headers,
        )

    async def parse(self, response: Response):
        async for item in AnimeItem.get_items(html=await response.text()):
            profile = item.profile
            player_aaaa = eval(re.search('{.*}', profile).group())
            item.episodes = re.findall('\d+', response.url)[2]
            encoded_url = player_aaaa['url']
            index_m3u8_url = unescape(
                base64_decode(encoded_url)
            )  # 目标网站的index.m3u8文件地址做了加密
            item._base_m3u8_url = index_m3u8_url[:-10]
            await self._mixed_m3u8_url_parse(index_m3u8_url, item)

            link_next = player_aaaa.get('link_next', None)
            if link_next:
                yield await self.have_next_page(link_next)
            yield item

    async def process_item(self, item: AnimeItem):
        resp = await self.request(item.mixed_m3u8_url, headers=self.headers).fetch()
        text = await resp.text()
        episodes = item.episodes
        folder_path = self.PATH / f'{episodes}'
        print('\033[0;32;40m写入mixed.m3u8\033[0m')
        await write(folder_path, text, 'mixed', 'm3u8', 'w+')
        urls = self._parse_mixed_m3u8(item)
        await Downloader(urls).download_ts_files(folder_path, episodes)
        self.logger.info(f"正在把第 {episodes} 集的ts文件转码成 mp4")
        await merge_ts2mp4(folder_path, episodes, self.del_ts)


if __name__ == '__main__':
    AnimeSpider.init('魔女之旅', ['https://www.mhyyy.com/play/166269-1-1.html']).start()



import os
import sys

from AnimeCrawler.command.run import CrawlerCommands

if sys.path[0] in ("", os.getcwd()):
    sys.path.pop(0)

if __package__ == "":
    path = os.path.dirname(os.path.dirname(__file__))
    sys.path.insert(0, path)

if __name__ == '__main__':
    CrawlerCommands().parse()
